import os
import requests
from dataclasses import dataclass
from datetime import datetime
from typing import Optional, List, Dict


API_TOKEN = os.getenv("CONNECTORS_AIRTABLE_TOKEN")


@dataclass
class AirtableRecord:
    id: str
    fields: dict
    created_time: datetime

    @staticmethod
    def from_dto(dto: dict):
        return AirtableRecord(
            id=dto["id"],
            fields=dto["fields"],
            created_time=datetime.fromisoformat(dto["createdTime"]),
        )


class AirtableConnector:
    def __init__(self):
        self.api_token = API_TOKEN
        self.requests = requests

    def make_request(
        self,
        *,
        base_id: str,
        table_name: str,
        method: str,
        record_id: str = None,
        data: Optional[dict] = None,
        query_params: Optional[Dict[str, List[str]]] = None,
    ) -> dict:
        url = f"https://api.airtable.com/v0/{base_id}/{table_name}"
        if record_id:
            url += f"/{record_id}"

        base_params = dict(
            method=method,
            url=url,
        )

        if query_params is not None:
            base_params["params"] = query_params

        if data is None:
            base_params["headers"] = {
                "Authorization": f"Bearer {self.api_token}",
            }
        else:
            base_params["headers"] = {
                "Authorization": f"Bearer {self.api_token}",
                "Content-Type": "application/json",
            }
            base_params["json"] = data

        response = self.requests.request(**base_params)

        return response.json()

    def create_record(self, base_id: str, table_name: str, data) -> str:
        data = {"fields": data}
        response = self.make_request(
            base_id=base_id, table_name=table_name, method="POST", data=data
        )
        return AirtableRecord.from_dto(response)

    def create_records(self, base_id: str, table_name: str, data) -> str:
        data = {"records": [{"fields": record} for record in data]}
        response = self.make_request(
            base_id=base_id, table_name=table_name, method="POST", data=data
        )
        return [AirtableRecord.from_dto(record) for record in response["records"]]

    def update_record(
        self, base_id: str, table_name: str, record_id: str, change: dict
    ) -> None:
        data = {"fields": change}
        self.make_request(
            base_id=base_id,
            table_name=table_name,
            method="PATCH",
            record_id=record_id,
            data=data,
        )

    def update_records(self, base_id: str, table_name: str, updates: dict) -> None:
        data = {"records": [{"fields": v, "id": k} for k, v in updates.items()]}
        self.make_request(
            base_id=base_id, table_name=table_name, method="PATCH", data=data
        )

    def upsert_records(
        self, base_id: str, table_name: str, field_name: str, upserts: dict
    ):
        data = {
            "performUpsert": {"fieldsToMergeOn": [field_name]},
            "records": [{"fields": {**v, field_name: k}} for k, v in upserts.items()],
        }
        self.make_request(
            base_id=base_id,
            table_name=table_name,
            method="PATCH",
            record_id=None,
            data=data,
        )

    def delete_record(self, base_id: str, table_name: str, record_id: str) -> None:
        self.make_request(
            base_id=base_id, table_name=table_name, method="DELETE", record_id=record_id
        )

    def delete_records(self, base_id: str, table_name: str, ids: List[str]):
        self.make_request(
            base_id=base_id,
            table_name=table_name,
            method="DELETE",
            query_params={"records": ids},
        )

    def get_record(self, base_id: str, table_name: str, record_id: str) -> dict:
        return self.make_request(
            base_id=base_id, table_name=table_name, method="GET", record_id=record_id
        )

    def list_records(self, base_id: str, table_name: str):
        records = self.make_request(
            base_id=base_id, table_name=table_name, method="GET"
        )["records"]
        for record in records:
            yield AirtableRecord.from_dto(record)


default_connector = AirtableConnector()


def create_record(base_id: str, table_name: str, data) -> str:
    return default_connector.create_record(base_id, table_name, data)


def create_records(base_id: str, table_name: str, data) -> str:
    return default_connector.create_records(base_id, table_name, data)


def update_record(base_id: str, table_name: str, record_id: str, change: dict) -> None:
    return default_connector.update_record(base_id, table_name, record_id, change)


def update_records(base_id: str, table_name: str, updates: dict) -> None:
    return default_connector.update_records(base_id, table_name, updates)


def upsert_records(base_id: str, table_name: str, field_name: str, upserts: dict):
    return default_connector.upsert_records(base_id, table_name, field_name, upserts)


def delete_record(base_id: str, table_name: str, record_id: str) -> None:
    return default_connector.delete_record(base_id, table_name, record_id)


def delete_records(base_id: str, table_name: str, ids: List[str]):
    return default_connector.delete_records(base_id, table_name, ids)


def get_record(base_id: str, table_name: str, record_id: str) -> dict:
    return default_connector.get_record(base_id, table_name, record_id)


def list_records(base_id: str, table_name: str):
    return default_connector.list_records(base_id, table_name)



from typing import Any, List, Optional

from gql import Client, gql
from gql.transport.aiohttp import AIOHTTPTransport
from pydantic.dataclasses import dataclass

MONDAY_URL = "https://api.monday.com/v2"
API_VERSION = "2023-10"

ITEM_FRAGMENT = """
    fragment itemFragment on Item {
        id
        name
        board {
            id
        }
        relative_link
        group {
            title
        }
        column_values {
            value
            text
            column {
                id
                title
                type
            }
        }
    }
"""


class AbstraMondayException(Exception):
    def __init__(self, message: str, original_exception: Optional[Exception] = None):
        self.message = f"Monday Connector Error: {message}.\n{original_exception}\nPlease contact Abstra support."


@dataclass
class SubscriptableDataclass:
    def __getitem__(self, key):
        return self.__dict__[key]

    def __setitem__(self, key, value):
        self.__dict__[key] = value


@dataclass
class ItemsPageDTO(SubscriptableDataclass):
    items: List["ItemDTO"]
    cursor: Optional[str]


@dataclass
class ColumnDTO(SubscriptableDataclass):
    id: str
    title: str
    type: str


@dataclass
class ColumnValueDTO(SubscriptableDataclass):
    value: Optional[str]
    text: Optional[str]
    column: ColumnDTO


@dataclass
class GroupDTO(SubscriptableDataclass):
    title: str


@dataclass
class BoardDTO(SubscriptableDataclass):
    id: str


@dataclass
class ItemDTO(SubscriptableDataclass):
    id: str
    name: str
    relative_link: str
    group: GroupDTO
    column_values: List[ColumnValueDTO]
    board: BoardDTO


class Item:
    id: str
    name: str
    board: BoardDTO
    relative_link: str
    group: GroupDTO
    column_values: List[ColumnValueDTO]
    api: "MondayConnector"

    def __init__(
        self,
        dto: ItemDTO,
        api: "MondayConnector",
    ):
        self.api = api
        self.__dict__.update(dto.__dict__)

    def __str__(self) -> str:
        return str(self.__dict__)

    @property
    def item_id(self):
        return self.id

    def set_column_value(
        self,
        column_title: Optional[str] = None,
        column_id: Optional[str] = None,
        value: Any = None,
    ) -> None:
        if not column_id and not column_title:
            raise AbstraMondayException(
                "Either column_title or column_id args must be provided"
            )

        if column_title:
            for column_value in self.column_values:
                if column_value.column.title == column_title:
                    column_id = column_value.column.id
                    break

        if not column_id:
            raise AbstraMondayException(
                f"Column with title '{column_title}' not found in item '{self.id}'\nColumns found: {[column.column.title for column in self.column_values]}"
            )

        self.api.update_item_column_value(
            item_id=str(self.id),
            column_id=column_id,
            board_id=str(self.board.id),
            value=value,
        )

        self.refetch()

    def get_column_value(self, column_title=None, column_id=None) -> Optional[str]:
        if column_id:
            for column_value in self.column_values:
                if column_value.column.id == column_id:
                    return column_value.text
            raise AbstraMondayException(
                f"Column with id '{column_id}' not found in item '{self.id}'\nColumns found: {[column.column.id for column in self.column_values]}"
            )

        if column_title:
            for column_value in self.column_values:
                if column_value.column.title == column_title:
                    return column_value.text
            raise AbstraMondayException(
                f"Column with title '{column_title}' not found in item '{self.id}'\nColumns found: {[column.column.title for column in self.column_values]}"
            )

        raise AbstraMondayException(
            "Either column_title or column_id args must be provided"
        )

    def refetch(self):
        item = self.api.fetch_item(self.id)
        self.__dict__.update(item.__dict__)


class MondayConnector:
    def __init__(self, api_key: str):
        self.api_key = api_key
        transport = AIOHTTPTransport(
            url=MONDAY_URL,
            headers={"Authorization": api_key, "API-Version": API_VERSION},
        )
        self.client = Client(transport=transport, fetch_schema_from_transport=True)

    def fetch_item_from_url(self, url: str) -> Optional["Item"]:
        def get_item_id_from_url(url: str):
            fragments = url.split("/")

            if fragments[-2] != "pulses":
                return None

            return fragments[-1]

        item_id = get_item_id_from_url(url)

        if not item_id:
            return None

        return self.fetch_item(int(item_id))

    def fetch_item(self, item_id) -> Optional["Item"]:
        query = gql(
            """
            query getItem($itemId: ID!) {
                items(ids: [$itemId]) {
                    ...itemFragment
                }
            }
        """
            + ITEM_FRAGMENT
        )

        params = {"itemId": item_id}

        response = self.client.execute(query, variable_values=params)

        if not response.get("items") and len(response["items"]) == 0:
            return None

        item_dto = response["items"][0]

        try:
            return Item(
                api=self,
                dto=ItemDTO(
                    id=item_dto["id"],
                    name=item_dto["name"],
                    relative_link=item_dto["relative_link"],
                    group=item_dto["group"],
                    column_values=item_dto["column_values"],
                    board=item_dto["board"],
                ),
            )
        except Exception as e:
            raise AbstraMondayException(
                f"Error while validating dto for item '{item_id}'",
                e,
            )

    def get_all_items(self, board_id: str) -> List["Item"]:
        items = []
        cursor = None
        while True:
            items_page = self._get_items_page_dto(board_id, cursor=cursor)
            cursor = items_page.cursor
            items.extend(
                [Item(api=self, dto=item_dto) for item_dto in items_page.items]
            )
            if cursor is None:
                break

        return items

    def _get_items_page_dto(
        self, board_id: str, cursor: Optional[str], limit=250
    ) -> ItemsPageDTO:
        query = gql(
            """
            query getItems($boardId: ID!, $limit: Int!, $cursor: String) {
                boards(ids: [$boardId]) {
                    items_page(limit: $limit, cursor: $cursor) {
                        cursor
                        items {
                            ...itemFragment
                        }
                    }
                }
            }
        """
            + ITEM_FRAGMENT
        )

        params = {"boardId": board_id, "limit": limit, "cursor": cursor}

        response = self.client.execute(query, variable_values=params)

        if not response.get("boards") and len(response["boards"]) == 0:
            raise AbstraMondayException(
                f"Error while fetching items for board '{board_id}'"
            )

        item_dtos = response["boards"][0]["items_page"]["items"]

        items = []

        for item_dto in item_dtos:
            try:
                items.append(
                    ItemDTO(
                        id=item_dto["id"],
                        name=item_dto["name"],
                        relative_link=item_dto["relative_link"],
                        group=item_dto["group"],
                        column_values=item_dto["column_values"],
                        board=item_dto["board"],
                    ),
                )
            except Exception as e:
                raise AbstraMondayException(
                    f"Error while validating dto for item '{item_dto['id']}'", e
                )

        return ItemsPageDTO(
            items=items, cursor=response["boards"][0]["items_page"]["cursor"]
        )

    def update_item_column_value(
        self, board_id: str, column_id: str, item_id: str, value: str
    ):
        query = gql(
            """
            mutation changeColumnValue(
                $boardId: ID!
                $itemId: ID, 
                $columnId: String!, 
                $value: String, 
            ) {
                change_simple_column_value(
                    item_id: $itemId, 
                    column_id: $columnId, 
                    board_id: $boardId,
                    value: $value, 
                ) {
                    id
                }
            }
        """
        )

        params = {
            "itemId": item_id,
            "columnId": column_id,
            "value": value,
            "boardId": board_id,
        }

        return self.client.execute(query, variable_values=params)



from .monday import (
    AbstraMondayException,
    SubscriptableDataclass,
    ItemsPageDTO,
    ColumnDTO,
    ColumnValueDTO,
    GroupDTO,
    BoardDTO,
    ItemDTO,
    Item,
    MondayConnector,
)


__all__ = [
    "AbstraMondayException",
    "SubscriptableDataclass",
    "ItemsPageDTO",
    "ColumnDTO",
    "ColumnValueDTO",
    "GroupDTO",
    "BoardDTO",
    "ItemDTO",
    "Item",
    "MondayConnector",
]



from typing import Optional
import requests
import os


def _is_email(channel: str) -> bool:
    return "@" in channel


def _get_user_id_from_email(email: str) -> str:
    token = os.getenv("SLACK_BOT_TOKEN")
    if token is None:
        raise Exception("SLACK_BOT_TOKEN env var is not set")

    response = requests.get(
        "https://slack.com/api/users.lookupByEmail",
        params={"email": email},
        headers={"Authorization": "Bearer " + token},
    )

    return response.json()["user"]["id"]


def send_message(msg: str, channel: Optional[str] = None):
    if channel is None:
        channel = os.getenv("SLACK_CHANNEL")

    if channel is None:
        raise Exception("channel not set")

    if _is_email(channel):
        channel = _get_user_id_from_email(channel)

    token = os.getenv("SLACK_BOT_TOKEN")
    if token is None:
        raise Exception("SLACK_BOT_TOKEN env var is not set")

    requests.post(
        "https://slack.com/api/chat.postMessage",
        json={
            "channel": channel,
            "text": msg,
        },
        headers={
            "Authorization": "Bearer " + token,
            "Content-type": "application/json; charset=utf-8",
        },
    )



from unittest import TestCase
from unittest.mock import Mock
from connectors.airtable.methods import AirtableConnector
from requests import Response


class TestAirtableConnector(TestCase):
    def setUp(self) -> None:
        self.connector = AirtableConnector()
        self.connector.api_token = "test_token"
        self.connector.requests = Mock()
        self.connector.requests.request.return_value = Response()
        self.connector.requests.request.return_value.headers["Content-Type"] = (
            "application/json"
        )

    def test_create_records(self):
        self.connector.requests.request.return_value._content = b"""{
            "records": [
                {
                "createdTime": "2022-09-12T21:03:48.000Z",
                "fields": {
                    "Address": "333 Post St",
                    "Name": "Union Square",
                    "Visited": true
                },
                "id": "rec560UJdUtocSouk"
                },
                {
                "createdTime": "2022-09-12T21:03:48.000Z",
                "fields": {
                    "Address": "1 Ferry Building",
                    "Name": "Ferry Building"
                },
                "id": "rec3lbPRG4aVqkeOQ"
                }
            ]
        }"""
        self.connector.create_records(
            "base_id",
            "table_name",
            [
                {"Address": "333 Post St", "Name": "Union Square", "Visited": True},
                {"Address": "1 Ferry Building", "Name": "Ferry Building"},
            ],
        )
        self.connector.requests.request.assert_called_with(
            method="POST",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={
                "Authorization": "Bearer test_token",
                "Content-Type": "application/json",
            },
            json={
                "records": [
                    {
                        "fields": {
                            "Address": "333 Post St",
                            "Name": "Union Square",
                            "Visited": True,
                        }
                    },
                    {
                        "fields": {
                            "Address": "1 Ferry Building",
                            "Name": "Ferry Building",
                        }
                    },
                ]
            },
        )

    def test_create_record(self):
        self.connector.requests.request.return_value._content = b"""{
            "createdTime": "2022-09-12T21:03:48.000Z",
            "fields": {
                "Address": "333 Post St",
                "Name": "Union Square",
                "Visited": true
            },
            "id": "rec560UJdUtocSouk"
        }"""

        self.connector.create_record(
            "base_id",
            "table_name",
            {"Address": "333 Post St", "Name": "Union Square", "Visited": True},
        )
        self.connector.requests.request.assert_called_with(
            method="POST",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={
                "Authorization": "Bearer test_token",
                "Content-Type": "application/json",
            },
            json={
                "fields": {
                    "Address": "333 Post St",
                    "Name": "Union Square",
                    "Visited": True,
                }
            },
        )

    def test_list_records(self):
        self.connector.requests.request.return_value._content = b"""{
            "records": [
                {
                    "createdTime": "2022-09-12T21:03:48.000Z",
                    "fields": {
                        "Address": "333 Post St",
                        "Name": "Union Square",
                        "Visited": true
                    },
                        "id": "rec560UJdUtocSouk"
                    },
                {
                    "createdTime": "2022-09-12T21:03:48.000Z",
                    "fields": {
                        "Address": "1 Ferry Building",
                        "Name": "Ferry Building"
                    },
                    "id": "rec3lbPRG4aVqkeOQ"
                }
            ]
        }"""

        yield from self.connector.list_records("base_id", "table_name")

        self.connector.requests.request.assert_called_with(
            method="GET",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={"Authorization": "Bearer test_token"},
        )

    def test_get_record(self):
        self.connector.requests.request.return_value._content = b"""{
            "createdTime": "2022-09-12T21:03:48.000Z",
            "fields": {
                "Address": "333 Post St",
                "Name": "Union Square",
                "Visited": true
            },
            "id": "rec560UJdUtocSouk"
        }"""

        self.connector.get_record("base_id", "table_name", "rec560UJdUtocSouk")
        self.connector.requests.request.assert_called_with(
            method="GET",
            url="https://api.airtable.com/v0/base_id/table_name/rec560UJdUtocSouk",
            headers={"Authorization": "Bearer test_token"},
        )

    def test_update_records(self):
        self.connector.requests.request.return_value._content = b"""{
            "records": [
                {
                "createdTime": "2022-09-12T21:03:48.000Z",
                "fields": {
                    "Address": "501 Twin Peaks Blvd",
                    "Name": "Twin Peaks",
                    "Visited": true
                },
                "id": "rec560UJdUtocSouk"
                },
                {
                "createdTime": "2022-09-12T21:03:48.000Z",
                "fields": {
                    "Address": "1 Ferry Building",
                    "Name": "Ferry Building",
                    "Visited": true
                },
                "id": "rec3lbPRG4aVqkeOQ"
                }
            ]
        }"""

        self.connector.update_records(
            "base_id",
            "table_name",
            {
                "rec560UJdUtocSouk": {
                    "Address": "501 Twin Peaks Blvd",
                    "Name": "Twin Peaks",
                    "Visited": True,
                },
                "rec3lbPRG4aVqkeOQ": {"Visited": True},
            },
        )
        self.connector.requests.request.assert_called_with(
            method="PATCH",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={
                "Authorization": "Bearer test_token",
                "Content-Type": "application/json",
            },
            json={
                "records": [
                    {
                        "fields": {
                            "Address": "501 Twin Peaks Blvd",
                            "Name": "Twin Peaks",
                            "Visited": True,
                        },
                        "id": "rec560UJdUtocSouk",
                    },
                    {"fields": {"Visited": True}, "id": "rec3lbPRG4aVqkeOQ"},
                ]
            },
        )

    def test_update_records_upsert(self):
        self.connector.requests.request.return_value._content = b"""
            {
            "createdRecords": [
                "recsHMqsp3GEm3lEi"
            ],
            "records": [
                {
                "createdTime": "2022-09-12T21:03:48.000Z",
                "fields": {
                    "Address": "501 Twin Peaks Blvd",
                    "Name": "Twin Peaks",
                    "Visited": true
                },
                "id": "rec560UJdUtocSouk"
                },
                {
                "createdTime": "2022-11-15T01:02:04.400Z",
                "fields": {
                    "Name": "New Park",
                    "Visited": true
                },
                "id": "recsHMqsp3GEm3lEi"
                }
            ],
            "updatedRecords": [
                "rec560UJdUtocSouk"
            ]
            }
        """

        self.connector.upsert_records(
            "base_id",
            "table_name",
            "Name",
            {
                "Twin Peaks": {"Address": "501 Twin Peaks Blvd", "Visited": True},
                "New Park": {"Visited": True},
            },
        )

        self.connector.requests.request.assert_called_with(
            method="PATCH",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={
                "Authorization": "Bearer test_token",
                "Content-Type": "application/json",
            },
            json={
                "performUpsert": {"fieldsToMergeOn": ["Name"]},
                "records": [
                    {
                        "fields": {
                            "Address": "501 Twin Peaks Blvd",
                            "Name": "Twin Peaks",
                            "Visited": True,
                        }
                    },
                    {"fields": {"Name": "New Park", "Visited": True}},
                ],
            },
        )

    def test_update_record(self):
        self.connector.requests.request.return_value._content = b"""{
            "createdTime": "2022-09-12T21:03:48.000Z",
            "fields": {
                "Address": "1 Ferry Building",
                "Name": "Ferry Building",
                "Visited": true
            },
            "id": "rec3lbPRG4aVqkeOQ"
            }
        """

        self.connector.update_record(
            "base_id",
            "table_name",
            "record_id",
            {"Address": "1 Ferry Building", "Name": "Ferry Building", "Visited": True},
        )

        self.connector.requests.request.assert_called_with(
            method="PATCH",
            url="https://api.airtable.com/v0/base_id/table_name/record_id",
            headers={
                "Authorization": "Bearer test_token",
                "Content-Type": "application/json",
            },
            json={
                "fields": {
                    "Address": "1 Ferry Building",
                    "Name": "Ferry Building",
                    "Visited": True,
                }
            },
        )

    def test_delete_records(self):
        self.connector.requests.request.return_value._content = b"""{
            "records": [
                {
                "deleted": true,
                "id": "rec560UJdUtocSouk"
                },
                {
                "deleted": true,
                "id": "rec3lbPRG4aVqkeOQ"
                }
            ]
        }"""

        self.connector.delete_records(
            "base_id", "table_name", ["rec560UJdUtocSouk", "rec3lbPRG4aVqkeOQ"]
        )

        self.connector.requests.request.assert_called_with(
            method="DELETE",
            url="https://api.airtable.com/v0/base_id/table_name",
            headers={"Authorization": "Bearer test_token"},
            params={"records": ["rec560UJdUtocSouk", "rec3lbPRG4aVqkeOQ"]},
        )

    def test_delete_record(self):
        self.connector.requests.request.return_value._content = b"""{
            "deleted": true,
            "id": "rec560UJdUtocSouk"
        }"""

        self.connector.delete_record("base_id", "table_name", "record_id")

        self.connector.requests.request.assert_called_with(
            method="DELETE",
            url="https://api.airtable.com/v0/base_id/table_name/record_id",
            headers={"Authorization": "Bearer test_token"},
        )



import unittest
import os

from connectors.monday import Item, MondayConnector

SUBDOMAIN = "abstra-cast"
API_KEY = os.environ.get("MONDAY_API_KEY", "")
BOARD_ID = "6255133301"
COLUMNS = [
    {
        "title": "Dropdown",
        "type": "dropdown",
        "options": ["Option 1", "Option 2", "Option 3"],
    },
    {
        "title": "Status",
        "type": "status",
        "options": ["Done", "Working on it", "Stuck", "Not Started"],
    },
    {
        "title": "Date",
        "type": "date",
    },
    {
        "type": "person",
        "title": "Person",
        "options": ["dev@abstra.app"],
    },
]


class TestMondayConnector(unittest.TestCase):
    def setUp(self):
        self.maxDiff = None
        self.monday = MondayConnector(API_KEY)

    def test_fetch_all_items(self):
        items = self.monday.get_all_items(BOARD_ID)

        self.assertIsInstance(items, list)
        self.assertGreater(len(items), 0)

    def test_fetch_item(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        self.assertIsInstance(item, Item)

    def test_fetch_item_from_url(self):
        items = self.monday.get_all_items(BOARD_ID)
        url = f"https://{SUBDOMAIN}.monday.com{items[0].relative_link}"

        item = self.monday.fetch_item_from_url(url)
        self.assertIsInstance(item, Item)

    def test_get_column_value(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        assert item

        item.get_column_value(column_title="Dropdown")
        item.get_column_value(column_title="Status")
        item.get_column_value(column_title="Date")
        item.get_column_value(column_title="Person")

        expected = {
            "Dropdown": item.column_values[0].text,
            "Status": item.column_values[1].text,
            "Date": item.column_values[2].text,
            "Person": item.column_values[3].text,
        }

        self.assertEqual(
            item.get_column_value(column_title="Dropdown"), expected["Dropdown"]
        )
        self.assertEqual(
            item.get_column_value(column_title="Status"), expected["Status"]
        )
        self.assertEqual(item.get_column_value(column_title="Date"), expected["Date"])
        self.assertEqual(
            item.get_column_value(column_title="Person"), expected["Person"]
        )

    def test_set_column_value_dropdown(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        assert item

        item.set_column_value(column_title="Dropdown", value="")
        self.assertEqual(item.get_column_value(column_title="Dropdown"), None)

        item.set_column_value(column_title="Dropdown", value=COLUMNS[0]["options"][0])
        self.assertEqual(
            item.get_column_value(column_title="Dropdown"), COLUMNS[0]["options"][0]
        )

    def test_set_column_value_status(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        assert item

        item.set_column_value(column_title="Status", value=COLUMNS[1]["options"][0])
        self.assertEqual(
            item.get_column_value(column_title="Status"), COLUMNS[1]["options"][0]
        )

        item.set_column_value(column_title="Status", value=COLUMNS[1]["options"][1])
        self.assertEqual(
            item.get_column_value(column_title="Status"), COLUMNS[1]["options"][1]
        )

    def test_set_column_value_date(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        assert item

        item.set_column_value(column_title="Date", value="")
        self.assertEqual(item.get_column_value(column_title="Date"), "")

        item.set_column_value(column_title="Date", value="2022-12-12")
        self.assertEqual(item.get_column_value(column_title="Date"), "2022-12-12")

    def test_set_column_value_person(self):
        items = self.monday.get_all_items(BOARD_ID)
        item = self.monday.fetch_item(items[0].id)

        assert item

        item.set_column_value(column_title="Person", value="")
        self.assertEqual(item.get_column_value(column_title="Person"), "")

        item.set_column_value(column_title="Person", value=COLUMNS[3]["options"][0])
        self.assertEqual(
            item.get_column_value(column_title="Person"), COLUMNS[3]["options"][0]
        )



from .methods import (
    create_record,
    create_records,
    update_record,
    update_records,
    upsert_records,
    delete_record,
    delete_records,
    get_record,
    list_records,
)

__all__ = [
    "create_record",
    "create_records",
    "update_record",
    "update_records",
    "upsert_records",
    "delete_record",
    "delete_records",
    "get_record",
    "list_records",
]
